// /******************************************************************************
//  * File: HandTrackingSubsystem.cs
//  * Copyright (c) 2023 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
//  *
//  * Confidential and Proprietary - Qualcomm Technologies, Inc.
//  *
//  ******************************************************************************/

using QCHT.Interactions.Hands;
using UnityEngine;

namespace QCHT.Interactions.Core
{
    public class HandTrackingSubsystem : XRHandTrackingSubsystem
    {
        public const string ID = "Qualcomm-HandTrackingSubsystem";

        private class HandTrackingProvider : XRHandTrackingSubsystem.Provider {

            public override void Start() {
                if (Status == QchtStatus.QchtStatusIdle) {
                    Status = QCHTOpenXRPlugin.StartHandTracking() == 0
                        ? QchtStatus.QchtStatusRunning
                        : QchtStatus.QchtStatusIdle;
                }
            } 

            public override void Stop() {
                if (Status == QchtStatus.QchtStatusRunning) {
                    Status = QCHTOpenXRPlugin.StopHandTracking() == 0
                        ? QchtStatus.QchtStatusIdle
                        : QchtStatus.QchtStatusRunning;
                }
            }

            public override void Destroy() => Stop();
            
            private QCHTOpenXRPlugin.XrPose[] _leftPoses = new QCHTOpenXRPlugin.XrPose[(int)XrHandJoint.XR_HAND_JOINT_MAX];
            private QCHTOpenXRPlugin.XrPose[] _rightPoses = new QCHTOpenXRPlugin.XrPose[(int)XrHandJoint.XR_HAND_JOINT_MAX];
            private float[] _radius = new float[(int) XrHandJoint.XR_HAND_JOINT_MAX];
            
            public override void GetHandData(bool isLeft, ref bool isTracked, ref int space, ref Pose rootPose, ref Pose[] joints, ref float scale, ref int gesture, ref float gestureRatio, ref float flipRatio) {
                if (Status != QchtStatus.QchtStatusRunning) return;
                ref var poses = ref isLeft ? ref _leftPoses : ref _rightPoses;
                space = (int) XrSpace.XR_HAND_WORLD;
                var handedness = isLeft ? QCHTOpenXRPlugin.XrHandSide.XR_HAND_LEFT : QCHTOpenXRPlugin.XrHandSide.XR_HAND_RIGHT;
                var result = QCHTOpenXRPlugin.TryLocateHandJoints(handedness, ref isTracked, poses, _radius, ref gesture, ref gestureRatio, ref flipRatio);
                if (result != 0)
                {
                    Status = QchtStatus.QchtStatusError;
                    return;
                }

                Status = QchtStatus.QchtStatusRunning;
                for (var i = 0; i < (int) XrHandJoint.XR_HAND_JOINT_MAX; i++)
                {
                    joints[i] = poses[i].ToPose();
                }

                scale = _radius[(int) XrHandJoint.XR_HAND_JOINT_WRIST];
                rootPose = joints[(int) XrHandJoint.XR_HAND_JOINT_WRIST];
            }
        }

        [RuntimeInitializeOnLoadMethod(RuntimeInitializeLoadType.SubsystemRegistration)]
        private static void RegisterDescriptor() {
            XRHandTrackingSubsystemDescriptor.Create(new XRHandTrackingSubsystemDescriptor.Cinfo {
                id = ID,
                providerType = typeof(HandTrackingProvider),
                subsystemTypeOverride = typeof(HandTrackingSubsystem)
            });
        }
    }
}