// /******************************************************************************
//  * File: QCHTOpenXRPlugin.cs
//  * Copyright (c) 2023 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
//  *
//  * Confidential and Proprietary - Qualcomm Technologies, Inc.
//  *
//  ******************************************************************************/

using System;
using System.Runtime.InteropServices;
using QCHT.Interactions.Hands;
using UnityEngine;

namespace QCHT.Interactions.Core
{
    internal static class QCHTOpenXRPlugin
    {
        [StructLayout(LayoutKind.Sequential)]
        public struct XrPose
        {
            private XrQuaternion _orientation;
            private XrVector3 _position;

            public XrPose(Pose pose)
            {
                _orientation = new XrQuaternion(pose.rotation);
                _position = new XrVector3(pose.position);
            }

            public XrPose(XrQuaternion orientation, XrVector3 position)
            {
                _orientation = orientation;
                _position = position;
            }

            public Pose ToPose()
            {
                return new Pose(_position.ToVector3(), _orientation.ToQuaternion());
            }
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct XrQuaternion
        {
            private float _x;
            private float _y;
            private float _z;
            private float _w;

            public XrQuaternion(Quaternion quaternion)
            {
                _x = quaternion.x;
                _y = quaternion.y;
                _z = -quaternion.z;
                _w = -quaternion.w;
            }

            public static XrQuaternion identity => new XrQuaternion(new Quaternion(0, 0, -0, -1));

            public Quaternion ToQuaternion()
            {
                return new Quaternion(_x, _y, -_z, -_w);
            }
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct XrVector3
        {
            private float _x;
            private float _y;
            private float _z;

            public XrVector3(Vector3 position)
            {
                _x = position.x;
                _y = position.y;
                _z = -position.z;
            }

            public static XrVector3 zero => new XrVector3(new Vector3(0, 0, -0));

            public Vector3 ToVector3()
            {
                return new Vector3(_x, _y, -_z);
            }
        }

        public enum XrHandSide
        {
            XR_HAND_LEFT = 1,
            XR_HAND_RIGHT = 2,
            XR_HAND_MAX_ENUM = 0x7FFFFFFF
        }

        private const string DllName = "QCHTOpenXRPlugin";

        [DllImport(DllName, EntryPoint = "GetInterceptedInstanceProcAddr")]
        internal static extern IntPtr GetInterceptedInstanceProcAddr(IntPtr func);

        [DllImport(DllName, EntryPoint = "StartHandTracking")]
        internal static extern int StartHandTracking();

        [DllImport(DllName, EntryPoint = "StopHandTracking")]
        internal static extern int StopHandTracking();

        [DllImport(DllName, EntryPoint = "SetAppSpace")]
        internal static extern int SetAppSpace(ulong xrSpace);

        [DllImport(DllName, EntryPoint = "TryLocateHandJoints")]
        internal static extern int TryLocateHandJoints(XrHandSide handedness,
            ref bool isTracked,
            [MarshalAs(UnmanagedType.LPArray, SizeConst = (int) XrHandJoint.XR_HAND_JOINT_MAX)]
            XrPose[] handPoses,
            [MarshalAs(UnmanagedType.LPArray, SizeConst = (int) XrHandJoint.XR_HAND_JOINT_MAX)]
            float[] radius,
            ref int gesture,
            ref float gestureRatio,
            ref float flipRatio);
    }
}