// /******************************************************************************
//  * File: XROriginUtility.cs
//  * Copyright (c) 2023 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
//  *
//  * Confidential and Proprietary - Qualcomm Technologies, Inc.
//  *
//  ******************************************************************************/

using UnityEngine;

#if UNITY_AR_FOUNDATION_LEGACY
using UnityEngine.XR.ARFoundation;
#endif

using Unity.XR.CoreUtils;

namespace QCHT.Interactions.Core
{
    public static class XROriginUtility
    {
        public static XROrigin FindXROrigin() => Object.FindObjectOfType<XROrigin>();

#if UNITY_AR_FOUNDATION_LEGACY
        public static ARSessionOrigin FindARSessionOrigin()  => Object.FindObjectOfType<ARSessionOrigin>();
#endif
        
        public static Camera GetOriginCamera() {
            Camera camera = null;
#if UNITY_AR_FOUNDATION_LEGACY
            camera =  FindARSessionOrigin()?.camera;
#endif         
            return camera ? camera : FindXROrigin()?.Camera;
        }
        
        public static Transform GetOriginTransform() {
            Transform transform = null;
#if UNITY_AR_FOUNDATION_LEGACY
            transform = FindARSessionOrigin()?.transform;
#endif
            return transform ? transform : FindXROrigin()?.transform;
        }
        
        public static Transform GetTrackableParent() {
            Transform parent = null;
#if UNITY_AR_FOUNDATION_LEGACY
            parent = FindARSessionOrigin()?.trackablesParent;
#endif
            return parent ? parent : FindXROrigin()?.TrackablesParent;
        }
    }
}