// /******************************************************************************
//  * File: Hand.cs
//  * Copyright (c) 2023 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
//  *
//  * Confidential and Proprietary - Qualcomm Technologies, Inc.
//  *
//  ******************************************************************************/

using UnityEngine;
using System.Collections.Generic;
using QCHT.Interactions.Core;
using QCHT.Interactions.Hands.VFF;

namespace QCHT.Interactions.Hands
{
    public class Hand : MonoBehaviour, IHandDataUpdater, IHandSkinnable, IHideable, IPhysible, IHandPoseOverrider
    {
        [SerializeField] private HandDriver _mainHandDriver;
        [SerializeField] private HandDriver _ghostHandDriver;
        [SerializeField] private HandPhysible _handPhysible;
        [SerializeField] private HandVisualizer _mainHandVisualizer;
        [SerializeField] private HandGhostVisualizer _ghostHandVisualizer;

        #region IHandDataUpdater
        
        public void UpdateData(XrSpace space, IReadOnlyList<Pose> joints, float scale) {
            _mainHandDriver.UpdateData(space, joints, scale);
            _ghostHandDriver.UpdateData(space, joints, scale);
        }
        
        #endregion
        
        #region IHandSkinnable
        
        public void SetSkin(HandSkin skin) {
            _mainHandVisualizer.SetSkin(skin);
            _ghostHandVisualizer.SetSkin(skin);
        }
        
        #endregion
        
        #region IHideable
        
        public void Show() {
            _mainHandVisualizer.Show();
            _ghostHandVisualizer.Show();
        }

        public void Hide() {
            _mainHandVisualizer.Hide();
            _ghostHandVisualizer.Hide();
        }
        
        #endregion

        #region IPhysible
        
        public void SetPhysible(bool active) {
            _handPhysible.SetPhysible(active);
        }
        
        #endregion

        #region IHandPoseOverrider

        public Pose RootPose => _mainHandDriver.RootPose;
        public float RootScale => _mainHandDriver.RootScale;
        
        public HandData? HandPoseOverride
        {
            get => _mainHandDriver.HandPoseOverride;
            set => _mainHandDriver.HandPoseOverride = value;
        }

        public HandMask? HandPoseMask
        {
            get => _mainHandDriver.HandPoseMask;
            set => _mainHandDriver.HandPoseMask = value;
        }
        
        #endregion
    }
}