// /******************************************************************************
//  * File: HandGhostVisualizer.cs
//  * Copyright (c) 2023 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
//  *
//  * Confidential and Proprietary - Qualcomm Technologies, Inc.
//  *
//  ******************************************************************************/

using System.Collections;
using UnityEngine;

namespace QCHT.Interactions.Hands
{
    public class HandGhostVisualizer : HandVisualizer
    {
        [SerializeField] private Transform mainRoot;
        [SerializeField, MinMax(0.01f, 1f)]
        private Vector2 distanceBlendAlpha = new Vector2(0.01f, 1f);
        [SerializeField] private Vector2 size;
        [SerializeField] private LineRenderer _lineRenderer;
        
        private GradientAlphaKey[] _alphaKeys;

        public Transform MainRoot {
            get => mainRoot;
            set => mainRoot = value;
        }

        private void Start() {
            if (_lineRenderer) {
                _alphaKeys = new GradientAlphaKey[_lineRenderer.colorGradient.alphaKeys.Length];
            
                for (var i = 0; i < _alphaKeys.Length; i++) {
                    _alphaKeys[i] = _lineRenderer.colorGradient.alphaKeys[i];
                }
            }
        }

        private void Update() {
            if (mainRoot == null)
                return;

            var dist = GetNormalizedDistanceToMain();
            SetAlpha(dist);
            
            if (_lineRenderer) {
                _lineRenderer.positionCount = 2;
                _lineRenderer.SetPosition(0, mainRoot.position);
                _lineRenderer.SetPosition(1, transform.position);
                var colorGradient = _lineRenderer.colorGradient;
                var newAlphaKeys = new GradientAlphaKey[_alphaKeys.Length];
                for (var i = 0; i < _alphaKeys.Length; i++) {
                    newAlphaKeys[i].alpha = _alphaKeys[i].alpha * dist;
                }
                colorGradient.SetKeys(colorGradient.colorKeys, newAlphaKeys);
                _lineRenderer.colorGradient = colorGradient;
                _lineRenderer.endWidth = Mathf.Lerp(size.x, size.y, dist);
            }
        }

        public void OnEnable() {
            _lineRenderer.enabled = true;
        }

        public void OnDisable() {
            _lineRenderer.enabled = false;
        }
        
        public new void SetSkin(HandSkin skin) {
            if (meshRenderer == null)
                return;
            
            meshRenderer.sharedMesh = skin.GhostMesh;
            meshRenderer.material = skin.GhostMaterial;
        }

        protected override IEnumerator FadeInAsync() {
            yield return Fade(GetNormalizedDistanceToMain());
        }
        
        private float GetNormalizedDistanceToMain() {
            var distance = Vector3.Distance(mainRoot.position, transform.position);
            distance = (distance - distanceBlendAlpha.x) / distanceBlendAlpha.y - distanceBlendAlpha.x;
            return Mathf.Clamp01(distance);
        }
    }
}