// /******************************************************************************
//  * File: QCHTHandMainToGhostLineRenderer.cs
//  * Copyright (c) 2023 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
//  *
//  * Confidential and Proprietary - Qualcomm Technologies, Inc.
//  *
//  ******************************************************************************/

using UnityEngine;

namespace QCHT.Interactions.Hands
{
    [RequireComponent(typeof(LineRenderer))]
    public class QCHTHandMainToGhostLineRenderer : MonoBehaviour
    {
        [SerializeField, MinMax(0.01f, 1f)]
        private Vector2 distanceBlendAlpha = new Vector2(0.01f, 1f);
        [SerializeField] private Transform mainRoot;
        [SerializeField] private Transform ghostRoot;
        [SerializeField] private Vector2 size;
        
        private LineRenderer _lineRenderer;

        private GradientAlphaKey[] _alphaKeys;
        
        private float _value;

        public float Value
        {
            set
            {
                _value = value;
                UpdateColor(_value);
            }
            get => _value;
        }
        
        private void Awake()
        {
            _lineRenderer = GetComponent<LineRenderer>();
            _alphaKeys = new GradientAlphaKey[_lineRenderer.colorGradient.alphaKeys.Length];
            
            for (var i = 0; i < _alphaKeys.Length; i++)
            {
                _alphaKeys[i] = _lineRenderer.colorGradient.alphaKeys[i];
            }
        }

        private void Update()
        {
            _lineRenderer.positionCount = 2;
            _lineRenderer.SetPosition(0, mainRoot.position);
            _lineRenderer.SetPosition(1, ghostRoot.position);
            var alpha = GetNormalizedDistanceInMinMaxRange();
            _lineRenderer.enabled = alpha > 0.01f;
            UpdateColor(alpha);
        }
        
        public void OnEnable()
        {
            _lineRenderer.enabled = true;
        }

        public void OnDisable()
        {
            _lineRenderer.enabled = false;
        }

        private void UpdateColor(float value)
        {
            var colorGradient = _lineRenderer.colorGradient;
            var newAlphaKeys = new GradientAlphaKey[_alphaKeys.Length];
            for (var i = 0; i < _alphaKeys.Length; i++)
            {
                newAlphaKeys[i].alpha = _alphaKeys[i].alpha * value;
            }
            colorGradient.SetKeys(colorGradient.colorKeys, newAlphaKeys);
            _lineRenderer.colorGradient = colorGradient;
            _lineRenderer.endWidth = Mathf.Lerp(size.x, size.y, value);
        }
        
        private float GetNormalizedDistanceInMinMaxRange()
        {
            var distance = Vector3.Distance(mainRoot.position, ghostRoot.position);
            distance = (distance - distanceBlendAlpha.x) / distanceBlendAlpha.y - distanceBlendAlpha.x;
            return Mathf.Clamp01(distance);
        }
    }
}