// /******************************************************************************
//  * File: HandPhysible.cs
//  * Copyright (c) 2023 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
//  *
//  * Confidential and Proprietary - Qualcomm Technologies, Inc.
//  *
//  ******************************************************************************/

using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace QCHT.Interactions.Hands.VFF
{
    public class HandPhysible : MonoBehaviour, IPhysible
    {
        [SerializeField] private HandPhysicsConfiguration configuration;
        [SerializeField] private List<HandPhysicsPart> physicsParts;

        private void OnValidate() =>
            physicsParts = physicsParts.Count == 0 ? GetComponentsInChildren<HandPhysicsPart>().ToList() : physicsParts;

        private void Awake() {
            foreach (var part in physicsParts) {
                var boneConfiguration = configuration.Standard;
                if (part.IsThumb) boneConfiguration = configuration.Thumb;
                if (part.IsRoot) boneConfiguration = configuration.Root; 
                part.Configuration = boneConfiguration;
            }
            
            if (physicsParts.Count == 0)
                physicsParts = GetComponentsInChildren<HandPhysicsPart>().ToList();
        }

        public void SetPhysible(bool active) {
            foreach (var part in physicsParts)
                part.SetPhysible(active);
        }
    }
}