// /******************************************************************************
//  * File: XRHandFilter.cs
//  * Copyright (c) 2023 Qualcomm Technologies, Inc. and/or its subsidiaries. All rights reserved.
//  *
//  * Confidential and Proprietary - Qualcomm Technologies, Inc.
//  *
//  ******************************************************************************/

using UnityEngine;
using UnityEngine.XR.Interaction.Toolkit;
using UnityEngine.XR.Interaction.Toolkit.Filtering;

namespace QCHT.Interactions.Hands
{
    public class XRHandFilter : MonoBehaviour, IXRSelectFilter
    {
        private XRBaseInteractable _interactable;
        public bool canProcess => isActiveAndEnabled;

        [SerializeField] private SelectHand _selectHand;
        [SerializeField] private SelectGesture _selectGesture;

        public void Awake()
        {
            _interactable = GetComponent<XRBaseInteractable>();
        }

        public void OnEnable()
        {
            _interactable.selectFilters.Add(this);
        }

        public void OnDisable()
        {
            _interactable.selectFilters.Remove(this);
        }

        public bool Process(IXRSelectInteractor interactor, IXRSelectInteractable interactable)
        {
            var xrBaseControllerInteractor = interactor as XRBaseControllerInteractor;
            if (xrBaseControllerInteractor == null) return false;

            var xrHandController = xrBaseControllerInteractor.xrController as XRHandController;
            var state = xrBaseControllerInteractor.xrController.currentControllerState as XRHandState;

            if (state == null || xrHandController == null) return true;

            return CheckGesture(state.isGrabbed) && CheckHand(xrHandController.Handedness);
        }

        private bool CheckGesture(bool isGrab) => _selectGesture switch
        {
            SelectGesture.Both => true,
            SelectGesture.Pinch => !isGrab,
            SelectGesture.Grab => isGrab,
            _ => true
        };

        private bool CheckHand(XrHandedness handedness) => _selectHand switch
        {
            SelectHand.Both => true,
            SelectHand.Left => handedness == XrHandedness.XR_HAND_LEFT,
            SelectHand.Right => handedness == XrHandedness.XR_HAND_RIGHT,
            _ => true
        };
    }

    internal enum SelectGesture
    {
        Pinch,
        Grab,
        Both
    }

    internal enum SelectHand
    {
        Both,
        Right,
        Left
    }
}